import os

from enum import Enum
from limecore.core.types import ARGV
from pathlib import Path
from typing import Any, List, Tuple
from yaml import safe_load

from .configuration import Configuration


class ConfigurationType(Enum):
    ARGV = 0
    ENV = 1
    FILE = 2


class ConfigurationLoader:
    def __init__(self):
        self._search_path: List[Tuple[ConfigurationType, Any]] = []

    def add_commandline_arguments(self, argv: ARGV):
        self._search_path.append((ConfigurationType.ARGV, argv))

    def add_environment_variables(self, prefix: str):
        prefix = prefix.upper() + ("_" if not prefix.endswith("_") else "")

        self._search_path.append((ConfigurationType.ENV, prefix))

    def add_file(self, path: Path):
        self._search_path.append((ConfigurationType.FILE, path))

    def load(self) -> Configuration:
        configuration = Configuration({})

        for configuration_type, args in self._search_path:
            if configuration_type == ConfigurationType.ARGV:
                configuration = self._load_argv(args, configuration=configuration)
            elif configuration_type == ConfigurationType.ENV:
                configuration = self._load_environment_variables(
                    args, configuration=configuration
                )
            elif configuration_type == ConfigurationType.FILE:
                configuration = self._load_file(args, configuration=configuration)
            else:
                raise NotImplementedError(configuration_type)

        return configuration

    def _load_argv(self, argv: ARGV, configuration: Configuration):
        for k, v in [i.split("=", maxsplit=1) for i in argv.config]:
            item = {}

            for i, l in enumerate(reversed(k.split("_"))):
                if i == 0:
                    item = {l.lower(): v}
                else:
                    item = {l.lower(): item}

            configuration = Configuration(item, base=configuration)

        return configuration

    def _load_environment_variables(self, prefix: str, configuration: Configuration):
        for k, v in os.environ.items():
            if k.startswith(prefix):
                item = {}

                for i, l in enumerate(reversed(k.split("_")[1:])):
                    if i == 0:
                        item = {l.lower(): v}
                    else:
                        item = {l.lower(): item}

                configuration = Configuration(item, base=configuration)

        return configuration

    def _load_file(self, path: Path, configuration: Configuration):
        if path.exists() and path.suffix == ".yaml":
            with open(path) as f:
                return Configuration(safe_load(f), base=configuration)
        else:
            return configuration



from typing import Any, Dict, List, Optional


class Configuration:
    def __init__(self, data: Dict[str, Any], base: Optional["Configuration"] = None):
        self._data = {}

        if base is not None:
            self._merge(base._data)

        self._merge(data)

    def get_bool(self, name: str) -> Optional[bool]:
        if name not in self._data:
            return None

        return bool(self._data[name])

    def get_int(self, name: str) -> Optional[int]:
        if name not in self._data:
            return None

        return int(self._data[name])

    def get_list(self, name: str) -> List["Configuration"]:
        if name not in self._data:
            return []

        return list(Configuration(s) for s in (self._data[name] or []))

    def get_string(self, name: str) -> Optional[str]:
        if name not in self._data:
            return None

        return str(self._data[name])

    def section(self, *names: str) -> "Configuration":
        data = self._data

        for name in names:
            if name not in data:
                return Configuration({})

            data = data[name]

        return Configuration(data)

    def _merge(self, data: Dict[str, Any]):
        def merge(source: Dict[str, Any], target: Dict[str, Any]):
            for k, v in source.items():
                if isinstance(v, dict):
                    node = target.setdefault(k, {})
                    merge(v, node)
                else:
                    target[k] = v

            return target

        self._data = merge(data, self._data)

    def __str__(self):
        return str(self._data)



from injector import Module as _Module, provider, singleton
from limecore.core.types import ARGV, CWD
from pathlib import Path

from .configuration import Configuration
from .configuration_loader import ConfigurationLoader


class Module(_Module):
    def __init__(
        self,
        name: str,
        enable_commandline_arguments: bool = True,
        enable_environment_variables: bool = True,
    ):
        self._enable_commandline_arguments = enable_commandline_arguments
        self._enable_environment_variables = enable_environment_variables
        self._name = name

    @singleton
    @provider
    def provide_configuration(self, argv: ARGV, cwd: CWD) -> Configuration:
        configuration_loader = ConfigurationLoader()
        configuration_loader.add_file(cwd.joinpath(f"{self._name}.yaml"))
        configuration_loader.add_file(Path(f"~/.{self._name}").expanduser())

        if self._enable_commandline_arguments:
            configuration_loader.add_commandline_arguments(argv)

        if self._enable_environment_variables:
            configuration_loader.add_environment_variables(self._name)

        return configuration_loader.load()



import git
import os
import setuptools


repository = git.Repo(".")

# Ensure that the Repo is clean
if repository.is_dirty():
    raise Exception("You have uncommitted local changes.")

# Find the current Commit (e.g., what we are building)
commit = repository.commit()

tags = list(filter(lambda t: t.commit == commit, repository.tags))
# Find a Tag corresponding to the current Commit, it one exists
if len(tags) == 1:
    tag = tags[0]
elif len(tags) > 1:
    raise NotImplementedError("many tags")
else:
    tag = None

# The Version is derived either from the Tag or the current Commit
if tag is not None:
    version = tag.name
else:
    version = commit.hexsha[0:7]

# Read the README
with open("README.rst", "r") as f:
    long_description = f.read()

# Read the Requirements
with open("requirements.txt") as f:
    required = f.read().splitlines()

packages = []
# Discover the Packages
for directory, subdirectories, files in os.walk("src"):
    subdirectories[:] = [
        d
        for d in subdirectories
        if d not in ["__pycache__"] and not d.endswith(".egg-info")
    ]

    print(directory, directory.split(os.path.sep))
    if any(files):
        packages.append(".".join(directory.split(os.path.sep)[1:]))

# Build the Distribution
setuptools.setup(
    name="limecore-core-configuration",
    version=version,
    # Description
    description="limecore: Core Configuration",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    # GitHub Link
    url="https://github.com/limecore/core-configuration",
    # Classifiers
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # Owner Information
    author="Daniel Bradberry",
    author_email="daniel@danielbradberry.com",
    # License
    license="MIT",
    # Packages
    package_dir={"": "src"},
    packages=packages,
    # Requirements
    install_requires=required,
    python_requires=">=3.6",
)
